/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <algorithm>

#include <toolkit-window.h>

// Don't want to include the actual window.h which otherwise will be indirectly included by adaptor.h.
#define DALI_WINDOW_H
#include <dali/integration-api/adaptors/adaptor.h>

#include <dali/integration-api/adaptors/scene-holder.h>

#include <toolkit-adaptor-impl.h>
#include <dali/integration-api/debug.h>
#include <dali/integration-api/scene.h>
#include <test-application.h>
#include <test-render-surface.h>

namespace Dali
{

namespace
{

///////////////////////////////////////////////////////////////////////////////
//
// LogFactoryStub
//
///////////////////////////////////////////////////////////////////////////////

class LogFactory : public LogFactoryInterface
{
public:
  LogFactory() = default;
  virtual ~LogFactory() = default;

private:
  void InstallLogFunction() const override
  {
    Dali::Integration::Log::InstallLogFunction( &TestApplication::LogMessage );
  }
};
LogFactory* gLogFactory = NULL; // For some reason, destroying this when the Adaptor is destroyed causes a crash in some test cases when running all of them.
} //unnamed namespace

namespace Internal
{
namespace Adaptor
{

///////////////////////////////////////////////////////////////////////////////
//
// Dali::Internal::Adaptor::Adaptor Stub
//
///////////////////////////////////////////////////////////////////////////////

Dali::Adaptor* gAdaptor = nullptr;

Dali::Adaptor& Adaptor::New()
{
  DALI_ASSERT_ALWAYS( ! gAdaptor );
  gAdaptor = new Dali::Adaptor;
  return *gAdaptor;
}

Dali::Adaptor& Adaptor::Get()
{
  DALI_ASSERT_ALWAYS( gAdaptor );
  return *gAdaptor;
}

Adaptor::Adaptor()
{
}

Adaptor::~Adaptor()
{
  gAdaptor = nullptr;
}

void Adaptor::Start( Dali::Window window )
{
  if ( window )
  {
    mWindows.push_back( window );
    mWindowCreatedSignal.Emit( window );
  }
}

Integration::Scene Adaptor::GetScene( Dali::Window window )
{
  return window.GetScene();
}

bool Adaptor::AddIdle( CallbackBase* callback, bool hasReturnValue )
{
  mCallbacks.PushBack( callback );
  return true;
}

void Adaptor::RemoveIdle( CallbackBase* callback )
{
  mCallbacks.Erase( std::find_if( mCallbacks.Begin(), mCallbacks.End(),
                                  [ &callback ] ( CallbackBase* current ) { return callback == current; } ) );
}

void Adaptor::RunIdles()
{
  for( auto& callback : mCallbacks )
  {
    CallbackBase::Execute( *callback );
  }

  mCallbacks.Clear();
}

Dali::RenderSurfaceInterface& Adaptor::GetSurface()
{
  DALI_ASSERT_ALWAYS( ! mWindows.empty() );

  return reinterpret_cast < Dali::RenderSurfaceInterface& >( mWindows.front().GetRenderSurface() );
}

Dali::WindowContainer Adaptor::GetWindows()
{
  return mWindows;
}

Dali::Adaptor::AdaptorSignalType& Adaptor::ResizedSignal()
{
  return mResizedSignal;
}

Dali::Adaptor::AdaptorSignalType& Adaptor::LanguageChangedSignal()
{
  return mLanguageChangedSignal;
}

Dali::Adaptor::WindowCreatedSignalType& Adaptor::WindowCreatedSignal()
{
  return mWindowCreatedSignal;
}

} // namespace Adaptor
} // namespace Internal

///////////////////////////////////////////////////////////////////////////////
//
// Dali::Adaptor Stub
//
///////////////////////////////////////////////////////////////////////////////

Adaptor::Adaptor()
: mImpl( new Internal::Adaptor::Adaptor )
{
}

Adaptor::~Adaptor()
{
  Internal::Adaptor::gAdaptor = nullptr;
  delete mImpl;
}

void Adaptor::Start()
{
}

void Adaptor::Pause()
{
}

void Adaptor::Resume()
{
}

void Adaptor::Stop()
{
}

bool Adaptor::AddIdle( CallbackBase* callback, bool hasReturnValue )
{
  return mImpl->AddIdle( callback, hasReturnValue );
}

void Adaptor::RemoveIdle( CallbackBase* callback )
{
  mImpl->RemoveIdle( callback );
}

void Adaptor::ReplaceSurface( Window window, Dali::RenderSurfaceInterface& surface )
{
}

void Adaptor::ReplaceSurface( Dali::Integration::SceneHolder window, Dali::RenderSurfaceInterface& surface )
{
}

Adaptor::AdaptorSignalType& Adaptor::ResizedSignal()
{
  return mImpl->ResizedSignal();
}

Adaptor::AdaptorSignalType& Adaptor::LanguageChangedSignal()
{
  return mImpl->LanguageChangedSignal();
}

Adaptor::WindowCreatedSignalType& Adaptor::WindowCreatedSignal()
{
  return mImpl->WindowCreatedSignal();
}

Dali::RenderSurfaceInterface& Adaptor::GetSurface()
{
  return mImpl->GetSurface();
}

Dali::WindowContainer Adaptor::GetWindows() const
{
  return mImpl->GetWindows();
}

Any Adaptor::GetNativeWindowHandle()
{
  Any window;
  return window;
}

void Adaptor::ReleaseSurfaceLock()
{
}

void Adaptor::SetRenderRefreshRate( unsigned int numberOfVSyncsPerRender )
{
}

Adaptor& Adaptor::Get()
{
  return Internal::Adaptor::Adaptor::Get();
}

bool Adaptor::IsAvailable()
{
  return Internal::Adaptor::gAdaptor;
}

void Adaptor::NotifySceneCreated()
{
}

void Adaptor::NotifyLanguageChanged()
{
}

void Adaptor::FeedTouchPoint( TouchPoint& point, int timeStamp )
{
}

void Adaptor::FeedWheelEvent( WheelEvent& wheelEvent )
{
}

void Adaptor::FeedKeyEvent( KeyEvent& keyEvent )
{
}

void Adaptor::SceneCreated()
{
}

const LogFactoryInterface& Adaptor::GetLogFactory()
{
  if( gLogFactory == NULL )
  {
    gLogFactory = new LogFactory;
  }
  return *gLogFactory;
}

} // namespace Dali
